######################################################
#
# * Copyright 1999 Peter Apian-Bennewitz, pab-opto
# * ALL RIGHTS RESERVED
# * 
# * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF PAB-OPTO
# * The copyright notice above does not evidence any  actual  or
# * intended  publication of this source code and material is an
# * unpublished work by Peter Apian-Bennewitz.  This  material
# * contains CONFIDENTIAL INFORMATION that is the property and a
# * trade secret of Peter Apian-Bennewitz. Any use,  duplication
# * or  disclosure  not  specifically  authorized  in writing by
# * Peter Apian-Bennewitz is  strictly  prohibited.  THE  RECEIPT  OR
# * POSSESSION  OF  THIS SOURCE CODE AND/OR INFORMATION DOES NOT
# * CONVEY ANY RIGHTS TO REPRODUCE, DISCLOSE OR  DISTRIBUTE  ITS
# * CONTENTS,  OR  TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT
# * MAY DESCRIBE, IN WHOLE OR IN PART.
#  
######################################################


#######################
#
#	file selector operating inside a given namespace
#
#	$Header: /users/apian/r/rshow/RCS/file_sel.tcl,v 1.24 2000/01/31 13:54:27 apian Exp $
#
#	called with the fully qualified namespace, e.g. ::.n0::viewfile
#
#######################

proc file_sel {ns dir mask p_callback} {

	namespace eval ::${ns} {
		variable ns [uplevel {concat $ns}]
		variable dir [uplevel {concat $dir}]
		variable mask [uplevel {concat $mask}]
		variable p_callback [uplevel {concat $p_callback}]

		variable filedir $dir
		variable newdir
		variable filemask $mask
		variable callback $p_callback 
		variable wgn 

		#
		#	set default values
		#
		if { ! [info exists readtext] } { set readtext "read" }
		if { ! [info exists writetext] } { set writetext "write" }
		if { ! [info exists id] } { set id "-none-" }
		if { ! [info exists name] } { variable name }


		#	convert namespace notation to widget name
		regsub -all "::" $ns "_" wgn2 
		set wgn .$wgn2

		#	test if widget already exists
		if { [ winfo exists ${wgn} ] } {
		  wm deiconify ${wgn}
		  raise ${wgn}
		  return
		}

		#	intermediate step for callback
		proc callme { par } {
			upvar callback callback
			upvar filedir filedir  name name fullname fullname
			set fullname "$filedir/$name"
			eval $callback $par
		}

		#
		#	define procedures for updates and filtering of filenames
		#
		proc chdir {nd} {
			upvar wgn wgn  filedir filedir
			if { "/" == [ string index $nd 0 ] } {
			  set filedir $nd
			  return 0
			}
			set p [split $filedir "/"]
			foreach atom [split $nd "/"] {
			  if { $atom == "." } { continue }
			  if { $atom == ".." } {
				if { [ llength $p ] <= 2 } {
					set filedir [pwd]
					set p [split $filedir "/"]
				}
				set lp [ expr [llength $p] - 1 ]
				set p [ lreplace $p $lp $lp ]
				continue
			  }
			  lappend p $atom
			}
			set filedir [ join $p "/" ]
		}

		proc file_refresh {} {
			upvar wgn wgn  filedir filedir  filemask filemask
			$wgn.fs.file.list delete 0 end
			foreach i [lsort [glob -nocomplain $filedir/$filemask ]] {	
				$wgn.fs.file.list insert end [ file tail $i ]
			}
		}

		proc dir_refresh {} {
			upvar wgn wgn  filedir filedir
			$wgn.ds.dir.list delete 0 end
			$wgn.ds.dir.list insert end ".."
			foreach i [lsort [glob -nocomplain $filedir/*/]] {	
			  $wgn.ds.dir.list insert end [ file tail $i ]
			}

		}

		#
		#	define geometry layout 
		#

		toplevel ${wgn}

		# 	file selection frame (includes filelist and mask entry )
		frame ${wgn}.fs
		# 	dir selection frame (includes dirlist and directory entry )
		frame ${wgn}.ds 

		#	scrolled dirlist
		#
		frame ${wgn}.ds.dir 
		listbox ${wgn}.ds.dir.list -width 10 -yscrollcommand "${wgn}.ds.dir.scroll set"
		scrollbar ${wgn}.ds.dir.scroll -orient vertical -command "${wgn}.ds.dir.list yview"
		grid ${wgn}.ds.dir.list	  -row 0 -column 0 -sticky nsew
		grid ${wgn}.ds.dir.scroll -row 0 -column 1 -sticky ns

		#	dirname label
		#
		frame ${wgn}.ds.dirname 
		entry ${wgn}.ds.dirname.entry -textvariable ::${ns}::newdir 
		grid ${wgn}.ds.dirname.entry -sticky ew

		#	scrolled filelist 
		#
		frame ${wgn}.fs.file
		listbox ${wgn}.fs.file.list -width 0 -yscrollcommand "${wgn}.fs.file.scroll set"
		scrollbar ${wgn}.fs.file.scroll -orient vertical -command "${wgn}.fs.file.list yview"
		grid ${wgn}.fs.file.list	-in ${wgn}.fs.file -row 0 -column 0 -sticky ewns
		grid ${wgn}.fs.file.scroll	-in ${wgn}.fs.file -row 0 -column 1 -sticky ns
		grid columnconfigure	${wgn}.fs.file 0 -weight 1 
		grid rowconfigure	${wgn}.fs.file 0 -weight 1

		#	filename mask entry
		#
		frame ${wgn}.mask
		label ${wgn}.mask.label -text "mask:"
		entry ${wgn}.mask.entry -textvariable ::${ns}::filemask  -width 10
		grid ${wgn}.mask.label ${wgn}.mask.entry 
		
		#	filename display
		#
		frame ${wgn}.name
		label ${wgn}.name.label -text "name:"
		entry ${wgn}.name.entry -textvariable ::${ns}::name
		grid ${wgn}.name.label ${wgn}.name.entry

		#	action buttons
		#
		frame ${wgn}.action
		button ${wgn}.action.cancel -text "cancel" -command "destroy ${wgn}"

		button ${wgn}.action.read  \
			-text [subst $[subst ${ns}::readtext]] \
			-command "[ namespace code {callme R} ]"

		button ${wgn}.action.write  \
			-text [subst $[subst ${ns}::writetext]] \
			-command "[ namespace code {callme W} ]"
		grid ${wgn}.action.read ${wgn}.action.write ${wgn}.action.cancel


		#	assemble file selector subframe
		#	(putting mask above file jeopardizes widget resize -!? )
		grid ${wgn}.fs.file -in ${wgn}.fs -sticky nesw 
		grid ${wgn}.mask -in ${wgn}.fs

		#	assemble dir selector subframe
		grid ${wgn}.ds.dir	-sticky nsew 
		grid ${wgn}.ds.dirname	-sticky ew

		#	assemble it all
		grid ${wgn}.ds  -in ${wgn} -row 0 -column 0 -sticky nsew
		grid ${wgn}.fs  -in ${wgn} -row 0 -column 1 -sticky nsew
		grid ${wgn}.name -columnspan 2
		grid ${wgn}.action -columnspan 2

		#
		#	define geometry resize behaviour
		#

		grid columnconfigure	${wgn}.fs 0 -weight 1 
		grid rowconfigure 	${wgn}.fs 0 -weight 1

		grid columnconfigure	${wgn}.ds 0 -weight 1 
		grid rowconfigure 	${wgn}.ds 0 -weight 1
		grid columnconfigure	${wgn}.ds.dir 0 -weight 1 
		grid rowconfigure	${wgn}.ds.dir 0 -weight 1
		grid columnconfigure	${wgn}.ds.dirname 0 -weight 1
		grid columnconfigure	${wgn}.ds.dirname.entry 0 -weight 1

		grid columnconfigure	${wgn} 0 -weight 1 
		grid columnconfigure	${wgn} 1 -weight 1 
		grid rowconfigure 	${wgn} 0 -weight 1

		#
		#	define event callbacks
		#

		#	return in mask entry
		bind ${wgn}.mask.entry <Return> \
			"[ namespace code { file_refresh } ]"

		#	click in file list
		bind ${wgn}.fs.file.list <ButtonRelease-1> \
			"[ namespace code { set name [selection get] } ] " 
		bind ${wgn}.fs.file.list <Double-1> \
			"[ namespace code { set name [selection get] } ] ; \
			[ namespace code {callme R} ]"

		#	click in dir list
		bind ${wgn}.ds.dir.list <Double-1> "
			[ namespace code {
					chdir [selection get];
					set newdir $filedir
					file_refresh;
					dir_refresh
			}]" 

		#	return in dir entry
		bind ${wgn}.ds.dirname.entry <Return> "
			[ namespace code {
					chdir $newdir
					file_refresh
					dir_refresh
			}]"

		#	return in name entry
		bind ${wgn}.name.entry <Return> "
			[ namespace code {
				if { [ file isdirectory $filedir/$name ] ||
				     [ file isdirectory $name ] } {
					chdir $name
					file_refresh
					dir_refresh
				} else {
					callme R;
				}
			} ] "


		file_refresh
		dir_refresh
		set newdir $filedir

	}
}

