######################################################
#
# * Copyright 1999 Peter Apian-Bennewitz, pab-opto
# * ALL RIGHTS RESERVED
# * 
# * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF PAB-OPTO
# * The copyright notice above does not evidence any  actual  or
# * intended  publication of this source code and material is an
# * unpublished work by Peter Apian-Bennewitz.  This  material
# * contains CONFIDENTIAL INFORMATION that is the property and a
# * trade secret of Peter Apian-Bennewitz. Any use,  duplication
# * or  disclosure  not  specifically  authorized  in writing by
# * Peter Apian-Bennewitz is  strictly  prohibited.  THE  RECEIPT  OR
# * POSSESSION  OF  THIS SOURCE CODE AND/OR INFORMATION DOES NOT
# * CONVEY ANY RIGHTS TO REPRODUCE, DISCLOSE OR  DISTRIBUTE  ITS
# * CONTENTS,  OR  TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT
# * MAY DESCRIBE, IN WHOLE OR IN PART.
#  
######################################################


#######################
#
#	pop-up clipping plane selector for instances,
#	on a per-window specefication
#
#	for purity, this should not be in instance-tcl code, as the
#	clipping is stored with the camera structure.
#	However, clipping is only used for (and is probably only useful for)
#	displaying radiance geometry, so it's here.
#
#######################


proc instance_geo_clip { ns base } {

    namespace eval ${ns} {

	variable ns [uplevel {concat $ns}]
	variable base [uplevel {concat $base}]
	set pdm $base.geoclip

	if { [ winfo exists $pdm ] } {
		wm deiconify $pdm
		raise $pdm
		return
	} else {
		toplevel $pdm
	}

	proc flip {} {
	  variable ns
	  variable base
	  upvar ${ns}::cpd qq;  set qq [expr -$qq]
	  upvar ${ns}::cpx qq;  set qq [expr -$qq]
	  upvar ${ns}::cpy qq;  set qq [expr -$qq]
	  upvar ${ns}::cpz qq;  set qq [expr -$qq]
	  $base.acker guitrig X
	}

	proc apply {} {
	  variable ns
	  variable base
	  ${ns}::style_toggle clipping
	  ${ns}::style_bit_assembly
	  $base.acker guitrig X
	}

	label $pdm.t0 -text "clipping plane parameters $base" 


	# box for xyz plane coordinates
	frame $pdm.fxyz -relief groove  

	label $pdm.fxyz.l1 -text "plane (x,y,z,d)"
	label $pdm.fxyz.l2 -text "flip side"
	label $pdm.fxyz.l3 -text "active"
	
	entry $pdm.fxyz.px -textvariable ${ns}::cpx -width 5
	entry $pdm.fxyz.py -textvariable ${ns}::cpy -width 5
	entry $pdm.fxyz.pz -textvariable ${ns}::cpz -width 5
	entry $pdm.fxyz.d  -textvariable ${ns}::cpd -width 5 
	button $pdm.fxyz.fs  -text "fs" -command "${ns}::flip"
	checkbutton $pdm.fxyz.a  -offvalue 0 -onvalue 1 -variable ${ns}::cpa

	grid $pdm.fxyz.l1 -column 1 -row 0 -columnspan 4
	grid $pdm.fxyz.l2 -column 5 -row 0
	grid $pdm.fxyz.l3 -column 6 -row 0

	grid $pdm.fxyz.px -column 1 -row 1 
	grid $pdm.fxyz.py -column 2 -row 1 
	grid $pdm.fxyz.pz -column 3 -row 1 
	grid $pdm.fxyz.d -column 4 -row 1 
	grid $pdm.fxyz.fs -column 5 -row 1 
	grid $pdm.fxyz.a -column 6 -row 1 

	# buttons
	button $pdm.apl -text "clip on/off"  -command "${ns}::apply"
	button $pdm.ca  -text "cancel" -command "destroy ${pdm}"

	grid $pdm.t0 -column 0 -row 0 -columnspan 2
	grid $pdm.fxyz -column 0 -row 1 -columnspan 2  -sticky w
	grid $pdm.apl -column 0 -row 6 
	grid $pdm.ca  -column 1 -row 6 

    }

}

#######################
#
#	instance_geo pulldown menu
#
#	$Header: /users/apian/r/rshow/RCS/instance_geo.tcl,v 1.28 2000/02/13 10:55:28 apian Exp $
#	sourced from gui1_add.tcl,
#	called from instance_geo_tcl.c after that function defined
#	namespace and variables for communication
#
#	variables used:
#	n0.stuff	integer with bits selecting primitive to draw
#	n0.style	integer with bits selecting wireframe/solid, texturing, clipping
#
#######################

proc instance_geo_setup { ns base } {

    namespace eval ${ns} {

	variable ns [uplevel {concat $ns}]
	variable base [uplevel {concat $base}]
	set pdm $base.frame_topbar.mb

	# X_qval is a temporary storage for the bitfields,
	# X_onval defines which primitive maps to which bit

	variable stuff_qval
	set stuff_onval(polygon)	0x03 
	set stuff_onval(instance)	0x08 
	set stuff_onval(cylinder)	0x10 
	set stuff_onval(disc)		0x20 
	set stuff_onval(cone)		0x40  
	set stuff_onval(sphere)		0x80 

	variable style_qval
	set style_onval(solid)		0x01
	set style_onval(texture)	0x02
	set style_onval(clipping)	0x10

	# options for sub-menus
	set stuff_opt(instance)	{-command "${ns}::stuff_bit_assembly" -selectcolor green -accelerator F2 }
	set stuff_opt(polygon)	{-command "${ns}::stuff_bit_assembly" -selectcolor green }
	set stuff_opt(cylinder)	{-command "${ns}::stuff_bit_assembly" -selectcolor green -accelerator F3}
	set stuff_opt(disc)	{-command "${ns}::stuff_bit_assembly" -selectcolor green -accelerator F3}
	set stuff_opt(cone)	{-command "${ns}::stuff_bit_assembly" -selectcolor green -accelerator F3}
	set stuff_opt(sphere)	{-command "${ns}::stuff_bit_assembly" -selectcolor green -accelerator F3}

	set style_opt(solid)	{-command "${ns}::style_bit_assembly" -selectcolor green -accelerator F5}
	set style_opt(texture)	{-command "${ns}::style_bit_assembly" -selectcolor green}
	set style_opt(clipping)	{-command "${ns}::style_bit_assembly" -selectcolor green}

	# set temporary values from 'stuff' variable, which
	# maybe set earlier by command line

	foreach q { instance polygon cylinder disc cone sphere} {
		if { $stuff & $stuff_onval($q) } {
			set stuff_qval($q) $stuff_onval($q)
		} else {
			set stuff_qval($q) 0
		}
	}

	foreach q { solid texture clipping } {
		if { $style & $style_onval($q) } {
			set style_qval($q) $style_onval($q)
		} else {
			set style_qval($q) 0
		}
	}

#
#	bit handling procedures
#

	# reassemble 'stuff' variable from temporary variables
	proc stuff_bit_assembly {} {
		variable stuff
		variable stuff_qval
		variable base
		set stuff 0
		foreach q {instance polygon cylinder disc cone sphere} {
			set stuff [expr $stuff + $stuff_qval($q)]
		}
		$base.acker guitrig X
	}

	# reassemble 'style' variable from temporary variables
	proc style_bit_assembly {} {
		variable style
		variable style_qval
		variable base
		set style 0
		foreach q {solid texture clipping} {
			set style [expr $style + $style_qval($q)]
		}
		$base.acker guitrig X
	}

	# stuff_toggle a temporary variable (e.g. by function key)
	proc stuff_toggle { varname } {
		variable stuff_qval
		variable stuff_onval
		upvar $varname var
		set stuff_qval($varname) [expr $stuff_qval($varname) ^ $stuff_onval($varname) ]
	}

	proc style_toggle { varname } {
		variable style_qval
		variable style_onval
		upvar $varname var
		set style_qval($varname) [expr $style_qval($varname) ^ $style_onval($varname) ]
	}

#
#	menu
#

	menubutton $pdm -text "instance_geo" -menu $pdm.m
	grid  $pdm -in ${base}.frame_topbar -row 1 -column 2

	menu $pdm.m 
	$pdm.m add cascade -label "display style"	-menu $pdm.m.d
	$pdm.m add cascade -label "geometric type"	-menu $pdm.m.gt
#	$pdm.m add command -label "culling ..."
#	$pdm.m add cascade -label "geometric name"
#	$pdm.m add cascade -label "material name"


#
#	sub-menus
#

	# define submenue with all primitives, use temporary variables
	# as variables for the buttons

	menu $pdm.m.gt
	foreach q { instance polygon cylinder disc cone sphere} {
		eval $pdm.m.gt add checkbutton\
			 -label $q \
			 -onvalue $stuff_onval($q) \
			 -variable ${ns}::stuff_qval($q) $stuff_opt($q)
	}

	menu $pdm.m.d
	foreach q { solid texture clipping } {
		eval $pdm.m.d add checkbutton\
			 -label $q \
			 -onvalue $style_onval($q) \
			 -variable ${ns}::style_qval($q) $style_opt($q)
	}
	$pdm.m.d add separator
	$pdm.m.d add command -label "clip parameters" -command "instance_geo_clip ${ns} ${base}"

#
#	function keys
#

	bind $base <F2> "[ namespace code {
		stuff_toggle instance
		stuff_bit_assembly
		}
	]"
	bind $base <F3> "[ namespace code {
		stuff_toggle cylinder
		stuff_toggle disc 
		stuff_toggle cone 
		stuff_toggle sphere
		stuff_bit_assembly
		}
	]"
	bind $base <F5> "[ namespace code {
		style_toggle solid
		style_bit_assembly
		}
	]"

   }

}
